function [LLROut] = SISO_LLR(DataIn,ppSNR,Modulation)

% QPSK mapping
Map_QPSK= ([1+i 1-i -1+i -1-i])/sqrt(2);

% 16QAM mapping
Map_16QAM= ((2/sqrt(10))* ...
    [ 0.5+0.5*i 0.5+1.5*i 0.5-0.5*i 0.5-1.5*i ...
    1.5+0.5*i 1.5+1.5*i 1.5-0.5*i 1.5-1.5*i ....
    -0.5+0.5*i -0.5+1.5*i -0.5-0.5*i -0.5-1.5*i ....
    -1.5+0.5*i -1.5+1.5*i -1.5-0.5*i -1.5-1.5*i]);

% 64QAM mapping 
Map_64QAM= ((2/sqrt(42))* ...
        [ 1.5+1.5*i 1.5+0.5*i 1.5+2.5*i 1.5+3.5*i ...
            1.5-1.5*i 1.5-0.5*i 1.5-2.5*i 1.5-3.5*i   ...
            0.5+1.5*i 0.5+0.5*i 0.5+2.5*i  0.5+3.5*i  ...
            0.5-1.5*i 0.5-0.5*i 0.5-2.5*i 0.5-3.5*i   ...
            2.5+1.5*i 2.5+0.5*i 2.5+2.5*i  2.5+3.5*i  ...
            2.5-1.5*i 2.5-0.5*i 2.5-2.5*i 2.5-3.5*i   ...
            3.5+1.5*i 3.5+0.5*i 3.5+2.5*i  3.5+3.5*i  ...
            3.5-1.5*i 3.5-0.5*i 3.5-2.5*i 3.5-3.5*i   ...
            -1.5+1.5*i -1.5+0.5*i -1.5+2.5*i  -1.5+3.5*i  ...
            -1.5-1.5*i -1.5-0.5*i -1.5-2.5*i -1.5-3.5*i   ...
            -0.5+1.5*i -0.5+0.5*i -0.5+2.5*i  -0.5+3.5*i  ...
            -0.5-1.5*i -0.5-0.5*i -0.5-2.5*i -0.5-3.5*i   ...
            -2.5+1.5*i -2.5+0.5*i -2.5+2.5*i  -2.5+3.5*i  ...
            -2.5-1.5*i -2.5-0.5*i -2.5-2.5*i -2.5-3.5*i   ...
            -3.5+1.5*i -3.5+0.5*i -3.5+2.5*i  -3.5+3.5*i  ...
            -3.5-1.5*i -3.5-0.5*i -3.5-2.5*i -3.5-3.5*i]); 




if Modulation==4
    Map=Map_QPSK;
elseif Modulation==16
    Map=Map_16QAM;
elseif Modulation==64
    Map=Map_64QAM;
end;


%compute corresponding bit format for each modulation
ModulationBits = [];
for MapCounter = log2(Modulation)-1:-1:0
    ModulationBits = [ModulationBits; rem(floor((1/pow2(MapCounter))*[0:Modulation-1]), 2)];
end

NumOfBits=log2(Modulation);
LLR=zeros(NumOfBits,size(DataIn,2));

for k=1:length(DataIn)
    
    % LLRs of the real part
    CostVector=abs(DataIn(k)*ones(1,Modulation)-Map).^2;
    CurrentLLR=zeros(NumOfBits,1);
    
    for k1=1:NumOfBits,
        CurrentLLR(k1)=-min(CostVector(find(ModulationBits(k1,:)==0)))+min(CostVector(find(ModulationBits(k1,:)==1)));
    end;
    CurrentLLR=CurrentLLR*ppSNR(k);
    
    LLR(:,k)=CurrentLLR;
end;

LLROut=LLR(:).';